/*
 * Copyright 2014 Univention GmbH
 *
 * http://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <http://www.gnu.org/licenses/>.
 */
/*global define*/

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/promise/all",
	"umc/dialog",
	"umc/widgets/Grid",
	"umc/widgets/Page",
	"umc/widgets/ExpandingTitlePane",
	"umc/widgets/Module",
	"umc/modules/webweaver/DetailPage",
	"umc/i18n!umc/modules/webweaver"
], function(declare, lang, array, all, dialog, Grid, Page, ExpandingTitlePane, Module, DetailPage, _) {
	return declare("umc.modules.webweaver", [ Module ], {

		idProperty: 'school',
		_grid: null,
		_page: null,
		_detailPage: null,

		postMixInProperties: function() {
			this.inherited(arguments);
			this.fixModuleStore();
		},

		fixModuleStore: function() {
			this.moduleStore.get = lang.hitch(this.moduleStore, function(id, handleErrors) {
				var data = {};
				data[this.idProperty] = id;
				return this._genericCmd('get', data, handleErrors);
			});
			this.moduleStore.put = lang.hitch(this.moduleStore, function(object, options, handleErrors) {
				return this._genericCmd('put', object, handleErrors);
			});
			this.moduleStore.remove = lang.hitch(this.moduleStore, function(object, options, handleErrors) {
				return this._genericCmd('delete', object, handleErrors);
			});
			this.moduleStore._genericMultiCmd = lang.hitch(this.moduleStore, function(type, params, handleErrors) {
				var args = arguments;
				var that = this;
				return all(array.map(params, function(param) {
					return that.inherited('_genericMultiCmd', args, [type, param, handleErrors]);
				}));
			});
			this.moduleStore._genericCmd = lang.hitch(this.moduleStore, function(type, param, handleErrors) {
				if (this._doingTransaction) {
					this._addTransactions(type, [param]);
				}
				else {
					return this._genericMultiCmd(type, [param], handleErrors).
						then(function(results) {
							if (results && results instanceof Array) {
								return results[0];
							}
						});
				}
			});
		},

		buildRendering: function() {
			this.inherited(arguments);

			this._page = new Page({
				headerText: _('Synchronisation with WebWeaver®'),
				helpText: _('Here you can enter one synchronisation account per school which was created on the respective WebWeaver® platform and equiped with administrator permissions for the respective school.') + ' ' + lang.replace(_('Via this account the synchronisation of users and classes with UCS@school takes place. Management system is UCS@school. Detailed information about about synchronisation can be found at {0}'), ['<a href="https://www.digionline.de/ucs" target="_blank">https://www.digionline.de/ucs</a>.'])
			});
			this.addChild(this._page);

			// generate the data grid
			this._grid = new Grid({
				actions: this.getGridActions(),
				columns: this.getGridColumns(),
				moduleStore: this.moduleStore,
				query: { no: 'thing' }
			});
			this._page.addChild(this._grid);

			// we need to call page's startup method manually as all widgets have
			// been added to the page container object
			this._page.startup();

			this._detailPage = new DetailPage({
				moduleStore: this.moduleStore
			});
			this.addChild(this._detailPage);

			this._detailPage.on('close', lang.hitch(this, function() {
				this.selectChild(this._page);
			}));
			this._detailPage.on('saved', lang.hitch(this, function() {
				this.selectChild(this._page);
				this.reloadGrid();
			}));
		},

		getGridColumns: function() {
			return [{
				name: 'displayName',
				label: _('School')
			}, {
				name: 'ServerLogin',
				label: _('Login for synchronisation')
			}, {
				name: 'ServerURL',
				label: _('WebWeaver® - URL')
			}, {
				name: 'error_email',
				label: _('E-Mail notifications'),
				formatter: function(value) {
					if (value === null || value === undefined) {
						return '';
					}
					return value ? _('Yes') : _('No');
				}
			}];
		},

		getGridActions: function() {
			return [{
				name: 'reload',
				label: _('Refresh'),
				isContextAction: false,
				isStandardAction: true,
				callback: lang.hitch(this, 'reloadGrid')
			}, {
				name: 'edit',
				label: _('Edit'),
				description: _('Edit the selected school'),
				iconClass: 'umcIconEdit',
				isStandardAction: true,
				isMultiAction: false,
				callback: lang.hitch(this, '_editObject')
			}, {
				name: 'delete',
				label: _('Delete'),
				description: _('Deleting the selected schools'),
				isStandardAction: true,
				isMultiAction: true,
				iconClass: 'umcIconDelete',
				canExecute: function(school) { return !!school.ServerURL; },
				callback: lang.hitch(this, '_deleteObjects')
			}, {
				name: 'resync',
				label: _('Resync'),
				description: _('Resyncing the settings'),
				isStandardAction: true,
				isMultiAction: false,
				canExecute: function(school) { return !!school.ServerURL; },
				callback: lang.hitch(this, '_resync')
			}];
		},

		_addObject: function() {
			this.selectChild(this._detailPage);
			this._detailPage.createSchool();
		},

		_editObject: function(ids, items) {
			this.selectChild(this._detailPage);
			this._detailPage.load(ids[0]);
		},

		_deleteObjects: function(ids, items) {
			dialog.confirm(_('Please confirm to delete the school "%s".', items[0].displayName), [{
				label: _('Delete'),
				callback: lang.hitch(this, '_remove', items)
			}, {
				label: _('Cancel'),
				'default': true
			}]);
		},

		_remove: function(items) {
			all(array.map(items, lang.hitch(this, function(school) {
				var remove = this.standbyDuring(this.moduleStore.remove({school: school.school}));
				remove.then(lang.hitch(this, function() {
					this.addNotification(_('The removal of school "%s" was successful.', school.displayName));
				}));
				return remove;
			}))).always(lang.hitch(this, 'reloadGrid'));
		},

		reloadGrid: function() {
			this._grid.filter({no: 'thing'});
		},

		_resync: function(ids, items) {
			array.map(items, lang.hitch(this, function(school) {
				var resync = this.standbyDuring(this.umcpCommand('webweaver/resync', {school: school.school}));
				resync.then(lang.hitch(this, function() {
					this.addNotification(_('The resynchronisation of school "%s" was successful.', school.displayName));
				}));
				return resync;
			}));
		}
	});
});
