/*
 * Copyright 2014 Univention GmbH
 *
 * http://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <http://www.gnu.org/licenses/>.
 */
/*global define*/

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"umc/dialog",
	"umc/widgets/Form",
	"umc/widgets/Page",
	"umc/widgets/TextBox",
	"umc/widgets/PasswordInputBox",
	"umc/widgets/CheckBox",
	"umc/widgets/StandbyMixin",
	"umc/i18n!umc/modules/webweaver"
], function(declare, lang, dialog, Form, Page, TextBox, PasswordInputBox, CheckBox, StandbyMixin, _) {
	return declare("umc.modules.webweaver.DetailPage", [Page, StandbyMixin], {
		moduleStore: null,
		_form: null,

		standbyOpacity: 1,
		postMixInProperties: function() {
			this.inherited(arguments);

			this.headerText = _('Define synchronisation account');
			this.helpText = _('Here you can define the synchronisation account for the WebWeaver® platform (e.g. synchronisation@...).') + ' ' + _('The account (login) have to exists already on the respective WebWeaver® platform and must be equiped with administrator permissions on school layer.');

			this.headerButtons = [{
				name: 'submit',
				label: _('Save'),
				iconClass: 'umcSaveIconWhite',
				callback: lang.hitch(this, function() {
					this._save(this._form.get('value'));
				})
			}, {
				name: 'close',
				iconClass: 'umcCloseIconWhite',
				label: _('Back to overview'),
				callback: lang.hitch(this, 'onClose')
			}];
		},

		buildRendering: function() {
			this.inherited(arguments);

			// create the form
			this._form = new Form({
				widgets: this.getWidgets(),
				layout: this.getLayout(),
				moduleStore: this.moduleStore,
				scrollable: true
			});

			this.addChild(this._form);

			this._form.on('submit', lang.hitch(this, '_save'));
		},

		getWidgets: function() {
			return [{
				type: TextBox,
				name: 'name',
				label: _('School'),
				disabled: true
			}, {
				type: TextBox,
				name: 'displayName',
				label: _('School'),
				disabled: true
			}, {
				type: TextBox,
				name: 'ServerLogin',
				label: _('Login')
			}, {
				type: PasswordInputBox,
				name: 'ServerPasswd',
				label: _('Password')
			}, {
				type: TextBox,
				name: 'ServerURL',
				label: _('URL of WebWeaver platform')
			}, {
				type: CheckBox,
				name: 'error_email',
				label: _('Notify via E-Mail on failures')
			}];
		},

		getLayout: function() {
			return [['displayName'], ['ServerURL'], ['ServerLogin', 'error_email'], ['ServerPasswd']];
		},

		_save: function(values) {
			this.standbyDuring(this.moduleStore.put(values, undefined, this._form)).then(lang.hitch(this, 'onSaved'));
		},

		load: function(id) {
			this._form.clearFormValues();
			this.standbyDuring(this._form.load(id));
		},

		onSaved: function() {
			// event stub
		},

		onClose: function() {
			// event stub
		}
	});
});
